@@ -1,28 +1,21 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-
-# Copyright (c) 2015-2018 PyTroll developers
-
-# Author(s):
-
-#   Martin Raspaud <martin.raspaud@smhi.se>
-#   David Hoese <david.hoese@ssec.wisc.edu>
-#   Adam Dybbroe <adam.dybbroe@smhi.se>
-
-# This program is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
-"""Base classes for composite objects.
-"""
+# Copyright (c) 2015-2019 Satpy developers
+#
+# This file is part of satpy.
+#
+# satpy is free software: you can redistribute it and/or modify it under the
+# terms of the GNU General Public License as published by the Free Software
+# Foundation, either version 3 of the License, or (at your option) any later
+# version.
+#
+# satpy is distributed in the hope that it will be useful, but WITHOUT ANY
+# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
+# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License along with
+# satpy.  If not, see <http://www.gnu.org/licenses/>.
+"""Base classes for composite objects."""
 
 import logging
 import os
@@ -30,30 +23,49 @@ import time
 import warnings
 from weakref import WeakValueDictionary
 
+import dask.array as da
 import numpy as np
 import six
 import xarray as xr
-import xarray.ufuncs as xu
-import dask.array as da
 import yaml
 
+try:
+    from yaml import UnsafeLoader
+except ImportError:
+    from yaml import Loader as UnsafeLoader
+
 from satpy.config import CONFIG_PATH, config_search_paths, recursive_dict_update
 from satpy.dataset import DATASET_KEYS, DatasetID, MetadataObject, combine_metadata
 from satpy.readers import DatasetDict
-from satpy.utils import sunzen_corr_cos, atmospheric_path_length_correction
+from satpy.utils import sunzen_corr_cos, atmospheric_path_length_correction, get_satpos
 from satpy.writers import get_enhanced_image
 from satpy import CHUNK_SIZE
 
+try:
+    from pyspectral.near_infrared_reflectance import Calculator
+except ImportError:
+    Calculator = None
+try:
+    from pyorbital.astronomy import sun_zenith_angle
+except ImportError:
+    sun_zenith_angle = None
+
+
 LOG = logging.getLogger(__name__)
 
+NEGLIBLE_COORDS = ['time']
+"""Keywords identifying non-dimensional coordinates to be ignored during composite generation."""
+
 
 class IncompatibleAreas(Exception):
     """Error raised upon compositing things of different shapes."""
+
     pass
 
 
 class IncompatibleTimes(Exception):
     """Error raised upon compositing things from different times."""
+
     pass
 
 
@@ -61,6 +73,7 @@ class CompositorLoader(object):
     """Read composites using the configuration files on disk."""
 
     def __init__(self, ppp_config_dir=None):
+        """Initialize the compositor loader."""
         if ppp_config_dir is None:
             ppp_config_dir = CONFIG_PATH
         self.modifiers = {}
@@ -81,6 +94,7 @@ class CompositorLoader(object):
         self._load_config(composite_configs)
 
     def get_compositor(self, key, sensor_names):
+        """Get the modifier for *sensor_names*."""
         for sensor_name in sensor_names:
             try:
                 return self.compositors[sensor_name][key]
@@ -89,6 +103,7 @@ class CompositorLoader(object):
         raise KeyError("Could not find compositor '{}'".format(key))
 
     def get_modifier(self, key, sensor_names):
+        """Get the modifier for *sensor_names*."""
         for sensor_name in sensor_names:
             try:
                 return self.modifiers[sensor_name][key]
@@ -115,6 +130,7 @@ class CompositorLoader(object):
 
                 Note that these dictionaries are copies of those cached in
                 this object.
+
         """
         comps = {}
         mods = {}
@@ -183,7 +199,7 @@ class CompositorLoader(object):
         conf = {}
         for composite_config in composite_configs:
             with open(composite_config) as conf_file:
-                conf = recursive_dict_update(conf, yaml.load(conf_file))
+                conf = recursive_dict_update(conf, yaml.load(conf_file, Loader=UnsafeLoader))
         try:
             sensor_name = conf['sensor_name']
         except KeyError:
@@ -214,6 +230,7 @@ class CompositorLoader(object):
 
 
 def check_times(projectables):
+    """Check that *projectables* have compatible times."""
     times = []
     for proj in projectables:
         try:
@@ -242,9 +259,9 @@ def check_times(projectables):
 def sub_arrays(proj1, proj2):
     """Substract two DataArrays and combine their attrs."""
     attrs = combine_metadata(proj1.attrs, proj2.attrs)
-    if (attrs.get('area') is None and
-            proj1.attrs.get('area') is not None and
-            proj2.attrs.get('area') is not None):
+    if (attrs.get('area') is None
+            and proj1.attrs.get('area') is not None
+            and proj2.attrs.get('area') is not None):
         raise IncompatibleAreas
     res = proj1 - proj2
     res.attrs = attrs
@@ -252,8 +269,10 @@ def sub_arrays(proj1, proj2):
 
 
 class CompositeBase(MetadataObject):
+    """Base class for all compositors and modifiers."""
 
     def __init__(self, name, prerequisites=None, optional_prerequisites=None, **kwargs):
+        """Initialise the compositor."""
         # Required info
         kwargs["name"] = name
         kwargs["prerequisites"] = prerequisites or []
@@ -261,17 +280,21 @@ class CompositeBase(MetadataObject):
         super(CompositeBase, self).__init__(**kwargs)
 
     def __call__(self, datasets, optional_datasets=None, **info):
+        """Generate a composite."""
         raise NotImplementedError()
 
     def __str__(self):
+        """Stringify the object."""
         from pprint import pformat
         return pformat(self.attrs)
 
     def __repr__(self):
+        """Represent the object."""
         from pprint import pformat
         return pformat(self.attrs)
 
     def apply_modifier_info(self, origin, destination):
+        """Apply the modifier info from *origin* to *destination*."""
         o = getattr(origin, 'attrs', origin)
         d = getattr(destination, 'attrs', destination)
         for k in DATASET_KEYS:
@@ -283,9 +306,28 @@ class CompositeBase(MetadataObject):
                 elif o.get(k) is not None:
                     d[k] = o[k]
 
-    def check_areas(self, data_arrays):
+    def match_data_arrays(self, data_arrays):
+        """Match data arrays so that they can be used together in a composite."""
+        self.check_geolocation(data_arrays)
+        return self.drop_coordinates(data_arrays)
+
+    def drop_coordinates(self, data_arrays):
+        """Drop neglible non-dimensional coordinates."""
+        new_arrays = []
+        for ds in data_arrays:
+            drop = [coord for coord in ds.coords
+                    if coord not in ds.dims and any([neglible in coord for neglible in NEGLIBLE_COORDS])]
+            if drop:
+                new_arrays.append(ds.drop(drop))
+            else:
+                new_arrays.append(ds)
+
+        return new_arrays
+
+    def check_geolocation(self, data_arrays):
+        """Check that the geolocations of the *data_arrays* are compatible."""
         if len(data_arrays) == 1:
-            return data_arrays
+            return
 
         if 'x' in data_arrays[0].dims and \
                 not all(x.sizes['x'] == data_arrays[0].sizes['x']
@@ -298,7 +340,7 @@ class CompositeBase(MetadataObject):
 
         areas = [ds.attrs.get('area') for ds in data_arrays]
         if all(a is None for a in areas):
-            return data_arrays
+            return
         elif any(a is None for a in areas):
             raise ValueError("Missing 'area' attribute")
 
@@ -307,7 +349,11 @@ class CompositeBase(MetadataObject):
                       "'{}'".format(self.attrs['name']))
             raise IncompatibleAreas("Areas are different")
 
-        return data_arrays
+    def check_areas(self, data_arrays):
+        """Check that the areas of the *data_arrays* are compatible."""
+        warnings.warn('satpy.composites.CompositeBase.check_areas is deprecated, use '
+                      'satpy.composites.CompositeBase.match_data_arrays instead')
+        return self.match_data_arrays(data_arrays)
 
 
 class SunZenithCorrectorBase(CompositeBase):
@@ -328,7 +374,8 @@ class SunZenithCorrectorBase(CompositeBase):
         super(SunZenithCorrectorBase, self).__init__(**kwargs)
 
     def __call__(self, projectables, **info):
-        projectables = self.check_areas(projectables)
+        """Generate the composite."""
+        projectables = self.match_data_arrays(projectables)
         vis = projectables[0]
         if vis.attrs.get("sunz_corrected"):
             LOG.debug("Sun zen correction already applied")
@@ -455,10 +502,12 @@ class EffectiveSolarPathLengthCorrector(SunZenithCorrectorBase):
 
 
 class PSPRayleighReflectance(CompositeBase):
+    """Pyspectral-based rayleigh corrector for visible channels."""
 
     _rayleigh_cache = WeakValueDictionary()
 
     def get_angles(self, vis):
+        """Get the sun and satellite angles fro the current dataarray."""
         from pyorbital.astronomy import get_alt_az, sun_zenith_angle
         from pyorbital.orbital import get_observer_look
 
@@ -466,12 +515,14 @@ class PSPRayleighReflectance(CompositeBase):
             chunks=vis.data.chunks)
 
         sunalt, suna = get_alt_az(vis.attrs['start_time'], lons, lats)
-        suna = xu.rad2deg(suna)
+        suna = np.rad2deg(suna)
         sunz = sun_zenith_angle(vis.attrs['start_time'], lons, lats)
+
+        sat_lon, sat_lat, sat_alt = get_satpos(vis)
         sata, satel = get_observer_look(
-            vis.attrs['satellite_longitude'],
-            vis.attrs['satellite_latitude'],
-            vis.attrs['satellite_altitude'],
+            sat_lon,
+            sat_lat,
+            sat_alt,
             vis.attrs['start_time'],
             lons, lats, 0)
         satz = 90 - satel
@@ -484,11 +535,11 @@ class PSPRayleighReflectance(CompositeBase):
         """
         from pyspectral.rayleigh import Rayleigh
         if not optional_datasets or len(optional_datasets) != 4:
-            vis, red = self.check_areas(projectables)
+            vis, red = self.match_data_arrays(projectables)
             sata, satz, suna, sunz = self.get_angles(vis)
             red.data = da.rechunk(red.data, vis.data.chunks)
         else:
-            vis, red, sata, satz, suna, sunz = self.check_areas(
+            vis, red, sata, satz, suna, sunz = self.match_data_arrays(
                 projectables + optional_datasets)
             sata, satz, suna, sunz = optional_datasets
             # get the dask array underneath
@@ -535,6 +586,7 @@ class PSPRayleighReflectance(CompositeBase):
 
 
 class NIRReflectance(CompositeBase):
+    """Get the reflective part of NIR bands."""
 
     def __call__(self, projectables, optional_datasets=None, **info):
         """Get the reflectance part of an NIR channel.
@@ -545,7 +597,7 @@ class NIRReflectance(CompositeBase):
         self._init_refl3x(projectables)
         _nir, _ = projectables
         refl = self._get_reflectance(projectables, optional_datasets) * 100
-        proj = xr.DataArray(refl.filled(np.nan), dims=_nir.dims,
+        proj = xr.DataArray(refl, dims=_nir.dims,
                             coords=_nir.coords, attrs=_nir.attrs)
 
         proj.attrs['units'] = '%'
@@ -554,13 +606,10 @@ class NIRReflectance(CompositeBase):
         return proj
 
     def _init_refl3x(self, projectables):
-        """Initiate the 3.x reflectance derivations."""
-        try:
-            from pyspectral.near_infrared_reflectance import Calculator
-        except ImportError:
+        """Initialize the 3.x reflectance derivations."""
+        if not Calculator:
             LOG.info("Couldn't load pyspectral")
-            raise
-
+            raise ImportError("No module named pyspectral.near_infrared_reflectance")
         _nir, _tb11 = projectables
         self._refl3x = Calculator(_nir.attrs['platform_name'], _nir.attrs['sensor'], _nir.attrs['name'])
 
@@ -583,14 +632,16 @@ class NIRReflectance(CompositeBase):
 
         # Check if the sun-zenith angle was provided:
         if sun_zenith is None:
-            from pyorbital.astronomy import sun_zenith_angle as sza
+            if sun_zenith_angle is None:
+                raise ImportError("No module named pyorbital.astronomy")
             lons, lats = _nir.attrs["area"].get_lonlats_dask(CHUNK_SIZE)
-            sun_zenith = sza(_nir.attrs['start_time'], lons, lats)
+            sun_zenith = sun_zenith_angle(_nir.attrs['start_time'], lons, lats)
 
         return self._refl3x.reflectance_from_tbs(sun_zenith, _nir, _tb11, tb_ir_co2=tb13_4)
 
 
 class NIREmissivePartFromReflectance(NIRReflectance):
+    """Get the emissive par of NIR bands."""
 
     def __call__(self, projectables, optional_datasets=None, **info):
         """Get the emissive part an NIR channel after having derived the reflectance.
@@ -605,7 +656,6 @@ class NIREmissivePartFromReflectance(NIRReflectance):
         # needs to be derived first in order to get the emissive part.
         _ = self._get_reflectance(projectables, optional_datasets)
         _nir, _ = projectables
-        raise NotImplementedError("This compositor wasn't fully converted to dask yet.")
         proj = xr.DataArray(self._refl3x.emissive_part_3x(), attrs=_nir.attrs,
                             dims=_nir.dims, coords=_nir.coords)
 
@@ -616,9 +666,13 @@ class NIREmissivePartFromReflectance(NIRReflectance):
 
 
 class PSPAtmosphericalCorrection(CompositeBase):
+    """Correct for atmospheric effects."""
 
     def __call__(self, projectables, optional_datasets=None, **info):
-        """Get the atmospherical correction. Uses pyspectral."""
+        """Get the atmospherical correction.
+
+        Uses pyspectral.
+        """
         from pyspectral.atm_correction_ir import AtmosphericalCorrection
 
         band = projectables[0]
@@ -628,13 +682,11 @@ class PSPAtmosphericalCorrection(CompositeBase):
         else:
             from pyorbital.orbital import get_observer_look
             lons, lats = band.attrs['area'].get_lonlats_dask(CHUNK_SIZE)
-
+            sat_lon, sat_lat, sat_alt = get_satpos(band)
             try:
-                dummy, satel = get_observer_look(band.attrs['satellite_longitude'],
-                                                 band.attrs[
-                                                     'satellite_latitude'],
-                                                 band.attrs[
-                                                     'satellite_altitude'],
+                dummy, satel = get_observer_look(sat_lon,
+                                                 sat_lat,
+                                                 sat_alt,
                                                  band.attrs['start_time'],
                                                  lons, lats, 0)
             except KeyError:
@@ -656,6 +708,7 @@ class PSPAtmosphericalCorrection(CompositeBase):
 
 
 class CO2Corrector(CompositeBase):
+    """Correct for CO2."""
 
     def __call__(self, projectables, optional_datasets=None, **info):
         """CO2 correction of the brightness temperature of the MSG 3.9um channel.
@@ -681,11 +734,13 @@ class CO2Corrector(CompositeBase):
 
 
 class DifferenceCompositor(CompositeBase):
+    """Make the difference of two data arrays."""
 
     def __call__(self, projectables, nonprojectables=None, **info):
+        """Generate the composite."""
         if len(projectables) != 2:
             raise ValueError("Expected 2 datasets, got %d" % (len(projectables),))
-        projectables = self.check_areas(projectables)
+        projectables = self.match_data_arrays(projectables)
         info = combine_metadata(*projectables)
         info['name'] = self.attrs['name']
 
@@ -695,6 +750,7 @@ class DifferenceCompositor(CompositeBase):
 
 
 class GenericCompositor(CompositeBase):
+    """Basic colored composite builder."""
 
     modes = {1: 'L', 2: 'LA', 3: 'RGB', 4: 'RGBA'}
 
@@ -741,7 +797,7 @@ class GenericCompositor(CompositeBase):
             # num may not be in `self.modes` so only check if we need to
             mode = self.modes[num]
         if len(projectables) > 1:
-            projectables = self.check_areas(projectables)
+            projectables = self.match_data_arrays(projectables)
             data = self._concat_datasets(projectables, mode)
             # Skip masking if user wants it or a specific alpha channel is given.
             if self.common_channel_mask and mode[-1] != 'A':
@@ -767,7 +823,10 @@ class GenericCompositor(CompositeBase):
         new_attrs.update({key: val
                           for (key, val) in attrs.items()
                           if val is not None})
+        resolution = new_attrs.get('resolution', None)
         new_attrs.update(self.attrs)
+        if resolution is not None:
+            new_attrs['resolution'] = resolution
         new_attrs["sensor"] = self._get_sensors(projectables)
         new_attrs["mode"] = mode
 
@@ -779,28 +838,35 @@ class FillingCompositor(GenericCompositor):
     """Make a regular RGB, filling the RGB bands with the first provided dataset's values."""
 
     def __call__(self, projectables, nonprojectables=None, **info):
+        """Generate the composite."""
+        projectables = self.check_areas(projectables)
         projectables[1] = projectables[1].fillna(projectables[0])
         projectables[2] = projectables[2].fillna(projectables[0])
         projectables[3] = projectables[3].fillna(projectables[0])
         return super(FillingCompositor, self).__call__(projectables[1:], **info)
 
 
-class RGBCompositor(GenericCompositor):
+class Filler(GenericCompositor):
+    """Fix holes in projectable 1 with data from projectable 2."""
 
     def __call__(self, projectables, nonprojectables=None, **info):
+        """Generate the composite."""
+        projectables = self.check_areas(projectables)
+        filled_projectable = projectables[0].fillna(projectables[1])
+        return super(Filler, self).__call__([filled_projectable], **info)
+
+
+class RGBCompositor(GenericCompositor):
+    """Make a composite from three color bands (deprecated)."""
+
+    def __call__(self, projectables, nonprojectables=None, **info):
+        """Generate the composite."""
         warnings.warn("RGBCompositor is deprecated, use GenericCompositor instead.", DeprecationWarning)
         if len(projectables) != 3:
             raise ValueError("Expected 3 datasets, got %d" % (len(projectables),))
         return super(RGBCompositor, self).__call__(projectables, **info)
 
 
-class BWCompositor(GenericCompositor):
-
-    def __call__(self, projectables, nonprojectables=None, **info):
-        warnings.warn("BWCompositor is deprecated, use GenericCompositor instead.", DeprecationWarning)
-        return super(BWCompositor, self).__call__(projectables, **info)
-
-
 class ColormapCompositor(GenericCompositor):
     """A compositor that uses colormaps."""
 
@@ -836,6 +902,7 @@ class ColorizeCompositor(ColormapCompositor):
     """A compositor colorizing the data, interpolating the palette colors when needed."""
 
     def __call__(self, projectables, **info):
+        """Generate the composite."""
         if len(projectables) != 2:
             raise ValueError("Expected 2 datasets, got %d" %
                              (len(projectables), ))
@@ -863,6 +930,7 @@ class PaletteCompositor(ColormapCompositor):
     """A compositor colorizing the data, not interpolating the palette colors."""
 
     def __call__(self, projectables, **info):
+        """Generate the composite."""
         if len(projectables) != 2:
             raise ValueError("Expected 2 datasets, got %d" % (len(projectables),))
 
@@ -897,7 +965,7 @@ class PaletteCompositor(ColormapCompositor):
 class DayNightCompositor(GenericCompositor):
     """A compositor that blends a day data with night data."""
 
-    def __init__(self, lim_low=85., lim_high=95., **kwargs):
+    def __init__(self, name, lim_low=85., lim_high=95., **kwargs):
         """Collect custom configuration values.
 
         Args:
@@ -908,10 +976,11 @@ class DayNightCompositor(GenericCompositor):
         """
         self.lim_low = lim_low
         self.lim_high = lim_high
-        super(DayNightCompositor, self).__init__(**kwargs)
+        super(DayNightCompositor, self).__init__(name, **kwargs)
 
     def __call__(self, projectables, **kwargs):
-        projectables = self.check_areas(projectables)
+        """Generate the composite."""
+        projectables = self.match_data_arrays(projectables)
 
         day_data = projectables[0]
         night_data = projectables[1]
@@ -967,8 +1036,7 @@ class DayNightCompositor(GenericCompositor):
 
 
 def enhance2dataset(dset):
-    """Apply enhancements to dataset *dset* and return the resulting data
-    array of the image."""
+    """Return the enhancemened to dataset *dset* as an array."""
     attrs = dset.attrs
     img = get_enhanced_image(dset)
     # Clip image data to interval [0.0, 1.0]
@@ -979,13 +1047,23 @@ def enhance2dataset(dset):
 
 
 def add_bands(data, bands):
-    """Add bands so that they match *bands*"""
+    """Add bands so that they match *bands*."""
     # Add R, G and B bands, remove L band
     if 'L' in data['bands'].data and 'R' in bands.data:
         lum = data.sel(bands='L')
-        new_data = xr.concat((lum, lum, lum), dim='bands')
-        new_data['bands'] = ['R', 'G', 'B']
-        data = new_data
+        # Keep 'A' if it was present
+        if 'A' in data['bands']:
+            alpha = data.sel(bands='A')
+            new_data = (lum, lum, lum, alpha)
+            new_bands = ['R', 'G', 'B', 'A']
+            mode = 'RGBA'
+        else:
+            new_data = (lum, lum, lum)
+            new_bands = ['R', 'G', 'B']
+            mode = 'RGB'
+        data = xr.concat(new_data, dim='bands', coords={'bands': new_bands})
+        data['bands'] = new_bands
+        data.attrs['mode'] = mode
     # Add alpha band
     if 'A' not in data['bands'].data and 'A' in bands.data:
         new_data = [data.sel(bands=band) for band in data['bands'].data]
@@ -998,6 +1076,7 @@ def add_bands(data, bands):
         alpha['bands'] = 'A'
         new_data.append(alpha)
         new_data = xr.concat(new_data, dim='bands')
+        new_data.attrs['mode'] = data.attrs['mode'] + 'A'
         data = new_data
 
     return data
@@ -1005,103 +1084,16 @@ def add_bands(data, bands):
 
 def zero_missing_data(data1, data2):
     """Replace NaN values with zeros in data1 if the data is valid in data2."""
-    nans = xu.logical_and(xu.isnan(data1), xu.logical_not(xu.isnan(data2)))
+    nans = np.logical_and(np.isnan(data1), np.logical_not(np.isnan(data2)))
     return data1.where(~nans, 0)
 
 
-class Airmass(GenericCompositor):
-
-    def __call__(self, projectables, *args, **kwargs):
-        """Make an airmass RGB image composite.
-
-        +--------------------+--------------------+--------------------+
-        | Channels           | Temp               | Gamma              |
-        +====================+====================+====================+
-        | WV6.2 - WV7.3      |     -25 to 0 K     | gamma 1            |
-        +--------------------+--------------------+--------------------+
-        | IR9.7 - IR10.8     |     -40 to 5 K     | gamma 1            |
-        +--------------------+--------------------+--------------------+
-        | WV6.2              |   243 to 208 K     | gamma 1            |
-        +--------------------+--------------------+--------------------+
-        """
-        warnings.warn("Airmass compositor is deprecated, use GenericCompositor "
-                      "with DifferenceCompositor instead.", DeprecationWarning)
-        ch1 = sub_arrays(projectables[0], projectables[1])
-        ch2 = sub_arrays(projectables[2], projectables[3])
-        res = super(Airmass, self).__call__((ch1, ch2,
-                                             projectables[0]),
-                                            *args, **kwargs)
-        return res
-
-
-class Convection(GenericCompositor):
-
-    def __call__(self, projectables, *args, **kwargs):
-        """Make a Severe Convection RGB image composite.
-
-        +--------------------+--------------------+--------------------+
-        | Channels           | Span               | Gamma              |
-        +====================+====================+====================+
-        | WV6.2 - WV7.3      |     -30 to 0 K     | gamma 1            |
-        +--------------------+--------------------+--------------------+
-        | IR3.9 - IR10.8     |      0 to 55 K     | gamma 1            |
-        +--------------------+--------------------+--------------------+
-        | IR1.6 - VIS0.6     |    -70 to 20 %     | gamma 1            |
-        +--------------------+--------------------+--------------------+
-        """
-        warnings.warn("Convection ompositor is deprecated, use GenericCompositor "
-                      "with DifferenceCompositor instead.", DeprecationWarning)
-
-        ch1 = sub_arrays(projectables[3], projectables[4])
-        ch2 = sub_arrays(projectables[2], projectables[5])
-        ch3 = sub_arrays(projectables[1], projectables[0])
-        res = super(Convection, self).__call__((ch1, ch2, ch3),
-                                               *args, **kwargs)
-        return res
-
-
-class Dust(GenericCompositor):
-
-    def __call__(self, projectables, *args, **kwargs):
-        """Make a dust (or fog or night_fog) RGB image composite.
-
-        Fog:
-        +--------------------+--------------------+--------------------+
-        | Channels           | Temp               | Gamma              |
-        +====================+====================+====================+
-        | IR12.0 - IR10.8    |     -4 to 2 K      | gamma 1            |
-        +--------------------+--------------------+--------------------+
-        | IR10.8 - IR8.7     |      0 to 6 K      | gamma 2.0          |
-        +--------------------+--------------------+--------------------+
-        | IR10.8             |   243 to 283 K     | gamma 1            |
-        +--------------------+--------------------+--------------------+
-
-        Dust:
-        +--------------------+--------------------+--------------------+
-        | Channels           | Temp               | Gamma              |
-        +====================+====================+====================+
-        | IR12.0 - IR10.8    |     -4 to 2 K      | gamma 1            |
-        +--------------------+--------------------+--------------------+
-        | IR10.8 - IR8.7     |     0 to 15 K      | gamma 2.5          |
-        +--------------------+--------------------+--------------------+
-        | IR10.8             |   261 to 289 K     | gamma 1            |
-        +--------------------+--------------------+--------------------+
-        """
-        warnings.warn("Dust compositor is deprecated, use GenericCompositor "
-                      "with DifferenceCompositor instead.", DeprecationWarning)
-
-        ch1 = sub_arrays(projectables[2], projectables[1])
-        ch2 = sub_arrays(projectables[1], projectables[0])
-        res = super(Dust, self).__call__((ch1, ch2,
-                                          projectables[1]),
-                                         *args, **kwargs)
-        return res
-
-
 class RealisticColors(GenericCompositor):
+    """Create a realistic colours composite for SEVIRI."""
 
     def __call__(self, projectables, *args, **kwargs):
-        projectables = self.check_areas(projectables)
+        """Generate the composite."""
+        projectables = self.match_data_arrays(projectables)
         vis06 = projectables[0]
         vis08 = projectables[1]
         hrv = projectables[2]
@@ -1126,8 +1118,9 @@ class RealisticColors(GenericCompositor):
 
 
 class CloudCompositor(GenericCompositor):
+    """Detect clouds based on thresholding and use it as a mask for compositing."""
 
-    def __init__(self, transition_min=258.15, transition_max=298.15,
+    def __init__(self, name, transition_min=258.15, transition_max=298.15,
                  transition_gamma=3.0, **kwargs):
         """Collect custom configuration values.
 
@@ -1142,10 +1135,10 @@ class CloudCompositor(GenericCompositor):
         self.transition_min = transition_min
         self.transition_max = transition_max
         self.transition_gamma = transition_gamma
-        super(CloudCompositor, self).__init__(**kwargs)
+        super(CloudCompositor, self).__init__(name, **kwargs)
 
     def __call__(self, projectables, **kwargs):
-
+        """Generate the composite."""
         data = projectables[0]
 
         # Default to rough IR thresholds
@@ -1193,6 +1186,7 @@ class RatioSharpenedRGB(GenericCompositor):
     """
 
     def __init__(self, *args, **kwargs):
+        """Instanciate the ration sharpener."""
         self.high_resolution_band = kwargs.pop("high_resolution_band", "red")
         if self.high_resolution_band not in ['red', 'green', 'blue', None]:
             raise ValueError("RatioSharpenedRGB.high_resolution_band must "
@@ -1222,7 +1216,7 @@ class RatioSharpenedRGB(GenericCompositor):
 
         new_attrs = {}
         if optional_datasets:
-            datasets = self.check_areas(datasets + optional_datasets)
+            datasets = self.match_data_arrays(datasets + optional_datasets)
             high_res = datasets[-1]
             p1, p2, p3 = datasets[:3]
             if 'rows_per_scan' in high_res.attrs:
@@ -1248,7 +1242,7 @@ class RatioSharpenedRGB(GenericCompositor):
             g = self._get_band(high_res, p2, 'green', ratio)
             b = self._get_band(high_res, p3, 'blue', ratio)
         else:
-            datasets = self.check_areas(datasets)
+            datasets = self.match_data_arrays(datasets)
             r, g, b = datasets[:3]
 
         # combine the masks
@@ -1309,12 +1303,11 @@ class SelfSharpenedRGB(RatioSharpenedRGB):
         new_G = G * ratio
         new_B = B * ratio
 
-
     """
 
     @staticmethod
     def four_element_average_dask(d):
-        """Average every 4 elements (2x2) in a 2D array"""
+        """Average every 4 elements (2x2) in a 2D array."""
         try:
             offset = d.attrs['area'].crop_offset
         except (KeyError, AttributeError):
@@ -1324,6 +1317,7 @@ class SelfSharpenedRGB(RatioSharpenedRGB):
         return xr.DataArray(res, attrs=d.attrs, dims=d.dims, coords=d.coords)
 
     def __call__(self, datasets, optional_datasets=None, **attrs):
+        """Generate the composite."""
         colors = ['red', 'green', 'blue']
         if self.high_resolution_band not in colors:
             raise ValueError("SelfSharpenedRGB requires at least one high resolution band, not "
@@ -1338,10 +1332,15 @@ class SelfSharpenedRGB(RatioSharpenedRGB):
 
 
 class LuminanceSharpeningCompositor(GenericCompositor):
+    """Create a high resolution composite by sharpening a low resolution using high resolution luminance.
+
+    This is done by converting to YCbCr colorspace, replacing Y, and convertin back to RGB.
+    """
 
     def __call__(self, projectables, *args, **kwargs):
+        """Generate the composite."""
         from trollimage.image import rgb2ycbcr, ycbcr2rgb
-        projectables = self.check_areas(projectables)
+        projectables = self.match_data_arrays(projectables)
         luminance = projectables[0].copy()
         luminance /= 100.
         # Limit between min(luminance) ... 1.0
@@ -1371,9 +1370,11 @@ class LuminanceSharpeningCompositor(GenericCompositor):
 
 
 class SandwichCompositor(GenericCompositor):
+    """Make a sandwich product."""
 
     def __call__(self, projectables, *args, **kwargs):
-        projectables = self.check_areas(projectables)
+        """Generate the composite."""
+        projectables = self.match_data_arrays(projectables)
         luminance = projectables[0]
         luminance /= 100.
         # Limit between min(luminance) ... 1.0
@@ -1383,3 +1384,99 @@ class SandwichCompositor(GenericCompositor):
         rgb_img = enhance2dataset(projectables[1])
         rgb_img *= luminance
         return super(SandwichCompositor, self).__call__(rgb_img, *args, **kwargs)
+
+
+class StaticImageCompositor(GenericCompositor):
+    """A compositor that loads a static image from disk."""
+
+    def __init__(self, name, filename=None, area=None, **kwargs):
+        """Collect custom configuration values.
+
+        Args:
+            filename (str): Filename of the image to load
+            area (str): Name of area definition for the image.  Optional
+                        for images with built-in area definitions (geotiff)
+        """
+        if filename is None:
+            raise ValueError("No image configured for static image compositor")
+        self.filename = filename
+        self.area = None
+        if area is not None:
+            from satpy.resample import get_area_def
+            self.area = get_area_def(area)
+
+        super(StaticImageCompositor, self).__init__(name, **kwargs)
+
+    def __call__(self, *args, **kwargs):
+        """Call the compositor."""
+        from satpy import Scene
+        scn = Scene(reader='generic_image', filenames=[self.filename])
+        scn.load(['image'])
+        img = scn['image']
+        # use compositor parameters as extra metadata
+        # most important: set 'name' of the image
+        img.attrs.update(self.attrs)
+        # Check for proper area definition.  Non-georeferenced images
+        # have None as .ndim
+        if img.area.ndim is None:
+            if self.area is None:
+                raise AttributeError("Area definition needs to be configured")
+            img.attrs['area'] = self.area
+        img.attrs['sensor'] = None
+        img.attrs['mode'] = ''.join(img.bands.data)
+        img.attrs.pop('modifiers', None)
+        img.attrs.pop('calibration', None)
+        # Add start time if not present in the filename
+        if 'start_time' not in img.attrs or not img.attrs['start_time']:
+            import datetime as dt
+            img.attrs['start_time'] = dt.datetime.utcnow()
+        if 'end_time' not in img.attrs or not img.attrs['end_time']:
+            import datetime as dt
+            img.attrs['end_time'] = dt.datetime.utcnow()
+
+        return img
+
+
+class BackgroundCompositor(GenericCompositor):
+    """A compositor that overlays one composite on top of another."""
+
+    def __call__(self, projectables, *args, **kwargs):
+        """Call the compositor."""
+        projectables = self.check_areas(projectables)
+
+        # Get enhanced datasets
+        foreground = enhance2dataset(projectables[0])
+        background = enhance2dataset(projectables[1])
+
+        # Adjust bands so that they match
+        # L/RGB -> RGB/RGB
+        # LA/RGB -> RGBA/RGBA
+        # RGB/RGBA -> RGBA/RGBA
+        foreground = add_bands(foreground, background['bands'])
+        background = add_bands(background, foreground['bands'])
+
+        # Get merged metadata
+        attrs = combine_metadata(foreground, background)
+
+        # Stack the images
+        if 'A' in foreground.mode:
+            # Use alpha channel as weight and blend the two composites
+            alpha = foreground.sel(bands='A')
+            data = []
+            # NOTE: there's no alpha band in the output image, it will
+            # be added by the data writer
+            for band in foreground.mode[:-1]:
+                fg_band = foreground.sel(bands=band)
+                bg_band = background.sel(bands=band)
+                chan = (fg_band * alpha + bg_band * (1 - alpha))
+                chan = xr.where(chan.isnull(), bg_band, chan)
+                data.append(chan)
+        else:
+            data = xr.where(foreground.isnull(), background, foreground)
+            # Split to separate bands so the mode is correct
+            data = [data.sel(bands=b) for b in data['bands']]
+
+        res = super(BackgroundCompositor, self).__call__(data, **kwargs)
+        res.attrs['area'] = attrs['area']
+
+        return res
