@@ -1,25 +1,20 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-
-# Copyright (c) 2015-2018 PyTroll developers
-
-# Author(s):
-
-#   Martin Raspaud <martin.raspaud@smhi.se>
-#   David Hoese <david.hoese@ssec.wisc.edu>
-
-# This program is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# Copyright (c) 2015-2018 Satpy developers
+#
+# This file is part of satpy.
+#
+# satpy is free software: you can redistribute it and/or modify it under the
+# terms of the GNU General Public License as published by the Free Software
+# Foundation, either version 3 of the License, or (at your option) any later
+# version.
+#
+# satpy is distributed in the hope that it will be useful, but WITHOUT ANY
+# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
+# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License along with
+# satpy.  If not, see <http://www.gnu.org/licenses/>.
 """Composite classes for the VIIRS instrument.
 """
 
@@ -30,7 +25,6 @@ import numpy as np
 import dask
 import dask.array as da
 import xarray as xr
-import xarray.ufuncs as xu
 
 from satpy.composites import CompositeBase, GenericCompositor
 from satpy.config import get_environ_ancpath
@@ -93,10 +87,10 @@ class ReflectanceCorrector(CompositeBase):
 
     def __call__(self, datasets, optional_datasets, **info):
         if not optional_datasets or len(optional_datasets) != 4:
-            vis = self.check_areas([datasets[0]])[0]
+            vis = self.match_data_arrays([datasets[0]])[0]
             sensor_aa, sensor_za, solar_aa, solar_za = self.get_angles(vis)
         else:
-            vis, sensor_aa, sensor_za, solar_aa, solar_za = self.check_areas(
+            vis, sensor_aa, sensor_za, solar_aa, solar_za = self.match_data_arrays(
                 datasets + optional_datasets)
             # get the dask array underneath
             sensor_aa = sensor_aa.data
@@ -481,7 +475,7 @@ class ERFDNB(CompositeBase):
             inner_sqrt = (output_dataset - min_val) / (max_val - min_val)
             # clip negative values to 0 before the sqrt
             inner_sqrt = inner_sqrt.where(inner_sqrt > 0, 0)
-            output_dataset.data = xu.sqrt(inner_sqrt).data
+            output_dataset.data = np.sqrt(inner_sqrt).data
 
         info = dnb_data.attrs.copy()
         info.update(self.attrs)
@@ -634,9 +628,9 @@ def local_histogram_equalization(data, mask_to_equalize, valid_data_mask=None, n
     total_cols = data.shape[1]
     tile_size = int((local_radius_px * 2.0) + 1.0)
     row_tiles = int(total_rows / tile_size) if (
-        total_rows % tile_size is 0) else int(total_rows / tile_size) + 1
+        (total_rows % tile_size) == 0) else int(total_rows / tile_size) + 1
     col_tiles = int(total_cols / tile_size) if (
-        total_cols % tile_size is 0) else int(total_cols / tile_size) + 1
+        (total_cols % tile_size) == 0) else int(total_cols / tile_size) + 1
 
     # an array of our distribution functions for equalization
     all_cumulative_dist_functions = [[]]
@@ -1004,7 +998,7 @@ def _linear_normalization_from_0to1(
     """
 
     LOG.debug(message)
-    if theoretical_min is not 0:
+    if theoretical_min != 0:
         data[mask] = data[mask] - theoretical_min
         theoretical_max = theoretical_max - theoretical_min
     data[mask] = data[mask] / theoretical_max
@@ -1039,7 +1033,7 @@ class NCCZinke(CompositeBase):
         dnb_data = dnb_data.copy() / unit_factor
 
         # convert to decimal instead of %
-        moon_illum_fraction = da.mean(datasets[3]) * 0.01
+        moon_illum_fraction = da.mean(datasets[3].data) * 0.01
 
         phi = da.rad2deg(da.arccos(2. * moon_illum_fraction - 1))
 
