@@ -1,20 +1,20 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
+# Copyright (c) 2016-2019 Satpy developers
 #
-# Copyright (c) 2016-2018 SatPy developers
+# This file is part of satpy.
 #
-# This program is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# satpy is free software: you can redistribute it and/or modify it under the
+# terms of the GNU General Public License as published by the Free Software
+# Foundation, either version 3 of the License, or (at your option) any later
+# version.
 #
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# satpy is distributed in the hope that it will be useful, but WITHOUT ANY
+# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
+# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU General Public License along with
+# satpy.  If not, see <http://www.gnu.org/licenses/>.
 """Advance Baseline Imager reader for the Level 1b format
 
 The files read by this reader are described in the official PUG document:
@@ -28,7 +28,6 @@ from datetime import datetime
 
 import numpy as np
 import xarray as xr
-import xarray.ufuncs as xu
 
 from pyresample import geometry
 from satpy.readers.file_handlers import BaseFileHandler
@@ -83,8 +82,11 @@ class NC_ABI_L1B(BaseFileHandler):
         # handle coordinates (and recursive fun)
         new_coords = {}
         # 'time' dimension causes issues in other processing
-        if 'time' in data.coords:
-            del data.coords['time']
+        # 'x_image' and 'y_image' are confusing to some users and unnecessary
+        # 'x' and 'y' will be overwritten by base class AreaDefinition
+        for coord_name in ('x_image', 'y_image', 'time', 'x', 'y'):
+            if coord_name in data.coords:
+                del data.coords[coord_name]
         if item in data.coords:
             self.coords[item] = data
         for coord_name in data.coords.keys():
@@ -124,11 +126,25 @@ class NC_ABI_L1B(BaseFileHandler):
                           'satellite_latitude': float(self['nominal_satellite_subpoint_lat']),
                           'satellite_longitude': float(self['nominal_satellite_subpoint_lon']),
                           'satellite_altitude': float(self['nominal_satellite_height'])})
+
+        # Add orbital parameters
+        projection = self.nc["goes_imager_projection"]
+        res.attrs['orbital_parameters'] = {
+            'projection_longitude': float(projection.attrs['longitude_of_projection_origin']),
+            'projection_latitude': float(projection.attrs['latitude_of_projection_origin']),
+            'projection_altitude': float(projection.attrs['perspective_point_height']),
+            'satellite_nominal_latitude': float(self['nominal_satellite_subpoint_lat']),
+            'satellite_nominal_longitude': float(self['nominal_satellite_subpoint_lon']),
+            'satellite_nominal_altitude': float(self['nominal_satellite_height']),
+            'yaw_flip': bool(self['yaw_flip_flag']),
+        }
+
         res.attrs.update(key.to_dict())
         # remove attributes that could be confusing later
         res.attrs.pop('_FillValue', None)
         res.attrs.pop('scale_factor', None)
         res.attrs.pop('add_offset', None)
+        res.attrs.pop('_Unsigned', None)
         res.attrs.pop('ancillary_variables', None)  # Can't currently load DQF
         # add in information from the filename that may be useful to the user
         for key in ('observation_type', 'scene_abbr', 'scan_mode', 'platform_shortname'):
@@ -136,6 +152,10 @@ class NC_ABI_L1B(BaseFileHandler):
         # copy global attributes to metadata
         for key in ('scene_id', 'orbital_slot', 'instrument_ID', 'production_site', 'timeline_ID'):
             res.attrs[key] = self.nc.attrs.get(key)
+        # only include these if they are present
+        for key in ('fusion_args',):
+            if key in self.nc.attrs:
+                res.attrs[key] = self.nc.attrs[key]
 
         return res
 
@@ -190,6 +210,7 @@ class NC_ABI_L1B(BaseFileHandler):
         res = data * factor
         res.attrs = data.attrs
         res.attrs['units'] = '1'
+        res.attrs['long_name'] = 'Bidirectional Reflectance'
         res.attrs['standard_name'] = 'toa_bidirectional_reflectance'
         return res
 
@@ -200,9 +221,10 @@ class NC_ABI_L1B(BaseFileHandler):
         bc1 = float(self["planck_bc1"])
         bc2 = float(self["planck_bc2"])
 
-        res = (fk2 / xu.log(fk1 / data + 1) - bc1) / bc2
+        res = (fk2 / np.log(fk1 / data + 1) - bc1) / bc2
         res.attrs = data.attrs
         res.attrs['units'] = 'K'
+        res.attrs['long_name'] = 'Brightness Temperature'
         res.attrs['standard_name'] = 'toa_brightness_temperature'
         return res
 
