@@ -1,27 +1,20 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
+# Copyright (c) 2014-2019 Satpy developers
 #
-# Copyright (c) 2014-2018 PyTroll developers
+# This file is part of satpy.
 #
-# Author(s):
+# satpy is free software: you can redistribute it and/or modify it under the
+# terms of the GNU General Public License as published by the Free Software
+# Foundation, either version 3 of the License, or (at your option) any later
+# version.
 #
-#   Adam.Dybbroe <adam.dybbroe@smhi.se>
-#   Cooke, Michael.C, UK Met Office
-#   Martin Raspaud <martin.raspaud@smhi.se>
+# satpy is distributed in the hope that it will be useful, but WITHOUT ANY
+# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
+# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #
-# This program is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
-
+# You should have received a copy of the GNU General Public License along with
+# satpy.  If not, see <http://www.gnu.org/licenses/>.
 """Advanced Himawari Imager (AHI) standard format data reader.
 
 References:
@@ -45,11 +38,12 @@ from datetime import datetime, timedelta
 import numpy as np
 import dask.array as da
 import xarray as xr
+import warnings
 
 from pyresample import geometry
 from satpy import CHUNK_SIZE
 from satpy.readers.file_handlers import BaseFileHandler
-from satpy.readers.utils import get_geostationary_mask, np2str
+from satpy.readers.utils import get_geostationary_mask, np2str, get_earth_radius
 
 AHI_CHANNEL_NAMES = ("1", "2", "3", "4", "5",
                      "6", "7", "8", "9", "10",
@@ -121,8 +115,8 @@ _NAV_INFO_TYPE = np.dtype([("hblock_number", "u1"),
                            ("SSP_longitude", "f8"),
                            ("SSP_latitude", "f8"),
                            ("distance_earth_center_to_satellite", "f8"),
-                           ("nadir_latitude", "f8"),
                            ("nadir_longitude", "f8"),
+                           ("nadir_latitude", "f8"),
                            ("sun_position", "f8", (3,)),
                            ("moon_position", "f8", (3,)),
                            ("spare", "S40"),
@@ -157,7 +151,10 @@ _IRCAL_INFO_TYPE = np.dtype([("c0_rad2tb_conversion", "f8"),
 # Visible, near-infrared band (Band No. 1 – 6)
 # (Band No. 1: backup operation (See Table 4 bb))
 _VISCAL_INFO_TYPE = np.dtype([("coeff_rad2albedo_conversion", "f8"),
-                              ("spare", "S104"),
+                              ("coeff_update_time", "f8"),
+                              ("cali_gain_count2rad_conversion", "f8"),
+                              ("cali_offset_count2rad_conversion", "f8"),
+                              ("spare", "S80"),
                               ])
 
 # 6 Inter-calibration information block
@@ -165,19 +162,19 @@ _INTER_CALIBRATION_INFO_TYPE = np.dtype([
     ("hblock_number", "u1"),
     ("blocklength", "<u2"),
     ("gsics_calibration_intercept", "f8"),
-    ("gsics_calibration_intercept_stderr", "f8"),
     ("gsics_calibration_slope", "f8"),
-    ("gsics_calibration_slope_stderr", "f8"),
     ("gsics_calibration_coeff_quadratic_term", "f8"),
-    ("gsics_calibration_coeff_quadratic_term_stderr",
-     "f8"),
+    ("gsics_std_scn_radiance_bias", "f8"),
+    ("gsics_std_scn_radiance_bias_uncertainty", "f8"),
+    ("gsics_std_scn_radiance", "f8"),
     ("gsics_correction_starttime", "f8"),
     ("gsics_correction_endtime", "f8"),
-    ("ancillary_text", "S64"),
-    ("spare", "S128"),
+    ("gsics_radiance_validity_upper_lim", "f4"),
+    ("gsics_radiance_validity_lower_lim", "f4"),
+    ("gsics_filename", "S128"),
+    ("spare", "S56"),
 ])
 
-
 # 7 Segment information block
 _SEGMENT_INFO_TYPE = np.dtype([
     ("hblock_number", "u1"),
@@ -208,7 +205,7 @@ _OBS_TIME_INFO_TYPE = np.dtype([
 # 10 Error information block
 _ERROR_INFO_TYPE = np.dtype([
     ("hblock_number", "u1"),
-    ("blocklength", "<u2"),
+    ("blocklength", "<u4"),
     ("number_of_error_info_data", "<u2"),
 ])
 
@@ -221,9 +218,47 @@ _SPARE_TYPE = np.dtype([
 
 
 class AHIHSDFileHandler(BaseFileHandler):
-    """AHI standard format reader."""
+    """AHI standard format reader
 
-    def __init__(self, filename, filename_info, filetype_info):
+    The AHI sensor produces data for some pixels outside the Earth disk (i,e:
+    atmospheric limb or deep space pixels).
+    By default, these pixels are masked out as they contain data of limited
+    or no value, but some applications do require these pixels.
+    It is therefore possible to override the default behaviour and perform no
+    masking of non-Earth pixels.
+
+    In order to change the default behaviour, use the 'mask_space' variable
+    as part of ``reader_kwargs`` upon Scene creation::
+
+        import satpy
+        import glob
+
+        filenames = glob.glob('*FLDK*.dat')
+        scene = satpy.Scene(filenames,
+                            reader='ahi_hsd',
+                            reader_kwargs={'mask_space':: False})
+        scene.load([0.6])
+
+    The AHI HSD data files contain multiple VIS channel calibration
+    coefficients. By default, the standard coefficients in header block 5
+    are used. If the user prefers the updated calibration coefficients then
+    they can pass calib_mode='update' when creating a scene::
+
+        import satpy
+        import glob
+
+        filenames = glob.glob('*FLDK*.dat')
+        scene = satpy.Scene(filenames,
+                            reader='ahi_hsd',
+                            reader_kwargs={'calib_mode':: 'update'})
+        scene.load([0.6])
+
+    By default these updated coefficients are not used.
+
+    """
+
+    def __init__(self, filename, filename_info, filetype_info,
+                 mask_space=True, calib_mode='nominal'):
         """Initialize the reader."""
         super(AHIHSDFileHandler, self).__init__(filename, filename_info,
                                                 filetype_info)
@@ -254,6 +289,13 @@ class AHIHSDFileHandler(BaseFileHandler):
         self.platform_name = np2str(self.basic_info['satellite'])
         self.observation_area = np2str(self.basic_info['observation_area'])
         self.sensor = 'ahi'
+        self.mask_space = mask_space
+
+        calib_mode_choices = ('NOMINAL', 'UPDATE')
+        if calib_mode.upper() not in calib_mode_choices:
+            raise ValueError('Invalid calibration mode: {}. Choose one of {}'.format(
+                calib_mode, calib_mode_choices))
+        self.calib_mode = calib_mode.upper()
 
     @property
     def start_time(self):
@@ -317,15 +359,34 @@ class AHIHSDFileHandler(BaseFileHandler):
         self.area = area
         return area
 
+    def _check_fpos(self, fp_, fpos, offset, block):
+        """Check file position matches blocksize"""
+        if (fp_.tell() + offset != fpos):
+            warnings.warn("Actual "+block+" header size does not match expected")
+        return
+
     def _read_header(self, fp_):
         """Read header"""
         header = {}
 
+        fpos = 0
         header['block1'] = np.fromfile(
             fp_, dtype=_BASIC_INFO_TYPE, count=1)
+        fpos = fpos + int(header['block1']['blocklength'])
+        self._check_fpos(fp_, fpos, 0, 'block1')
+        fp_.seek(fpos, 0)
         header["block2"] = np.fromfile(fp_, dtype=_DATA_INFO_TYPE, count=1)
+        fpos = fpos + int(header['block2']['blocklength'])
+        self._check_fpos(fp_, fpos, 0, 'block2')
+        fp_.seek(fpos, 0)
         header["block3"] = np.fromfile(fp_, dtype=_PROJ_INFO_TYPE, count=1)
+        fpos = fpos + int(header['block3']['blocklength'])
+        self._check_fpos(fp_, fpos, 0, 'block3')
+        fp_.seek(fpos, 0)
         header["block4"] = np.fromfile(fp_, dtype=_NAV_INFO_TYPE, count=1)
+        fpos = fpos + int(header['block4']['blocklength'])
+        self._check_fpos(fp_, fpos, 0, 'block4')
+        fp_.seek(fpos, 0)
         header["block5"] = np.fromfile(fp_, dtype=_CAL_INFO_TYPE, count=1)
         logger.debug("Band number = " +
                      str(header["block5"]['band_number'][0]))
@@ -336,13 +397,22 @@ class AHIHSDFileHandler(BaseFileHandler):
             cal = np.fromfile(fp_, dtype=_VISCAL_INFO_TYPE, count=1)
         else:
             cal = np.fromfile(fp_, dtype=_IRCAL_INFO_TYPE, count=1)
+        fpos = fpos + int(header['block5']['blocklength'])
+        self._check_fpos(fp_, fpos, 0, 'block5')
+        fp_.seek(fpos, 0)
 
         header['calibration'] = cal
 
         header["block6"] = np.fromfile(
             fp_, dtype=_INTER_CALIBRATION_INFO_TYPE, count=1)
+        fpos = fpos + int(header['block6']['blocklength'])
+        self._check_fpos(fp_, fpos, 0, 'block6')
+        fp_.seek(fpos, 0)
         header["block7"] = np.fromfile(
             fp_, dtype=_SEGMENT_INFO_TYPE, count=1)
+        fpos = fpos + int(header['block7']['blocklength'])
+        self._check_fpos(fp_, fpos, 0, 'block7')
+        fp_.seek(fpos, 0)
         header["block8"] = np.fromfile(
             fp_, dtype=_NAVIGATION_CORRECTION_INFO_TYPE, count=1)
         # 8 The navigation corrections:
@@ -355,7 +425,9 @@ class AHIHSDFileHandler(BaseFileHandler):
         corrections = []
         for i in range(ncorrs):
             corrections.append(np.fromfile(fp_, dtype=dtype, count=1))
-        fp_.seek(40, 1)
+        fpos = fpos + int(header['block8']['blocklength'])
+        self._check_fpos(fp_, fpos, 40, 'block8')
+        fp_.seek(fpos, 0)
         header['navigation_corrections'] = corrections
         header["block9"] = np.fromfile(fp_,
                                        dtype=_OBS_TIME_INFO_TYPE,
@@ -372,7 +444,9 @@ class AHIHSDFileHandler(BaseFileHandler):
                                                dtype=dtype,
                                                count=1))
         header['observation_time_information'] = lines_and_times
-        fp_.seek(40, 1)
+        fpos = fpos + int(header['block9']['blocklength'])
+        self._check_fpos(fp_, fpos, 40, 'block9')
+        fp_.seek(fpos, 0)
 
         header["block10"] = np.fromfile(fp_,
                                         dtype=_ERROR_INFO_TYPE,
@@ -387,9 +461,14 @@ class AHIHSDFileHandler(BaseFileHandler):
         for i in range(num_err_info_data):
             err_info_data.append(np.fromfile(fp_, dtype=dtype, count=1))
         header['error_information_data'] = err_info_data
-        fp_.seek(40, 1)
+        fpos = fpos + int(header['block10']['blocklength'])
+        self._check_fpos(fp_, fpos, 40, 'block10')
+        fp_.seek(fpos, 0)
 
-        np.fromfile(fp_, dtype=_SPARE_TYPE, count=1)
+        header["block11"] = np.fromfile(fp_, dtype=_SPARE_TYPE, count=1)
+        fpos = fpos + int(header['block11']['blocklength'])
+        self._check_fpos(fp_, fpos, 0, 'block11')
+        fp_.seek(fpos, 0)
 
         return header
 
@@ -425,26 +504,45 @@ class AHIHSDFileHandler(BaseFileHandler):
         # Calibrate
         res = self.calibrate(res, key.calibration)
 
+        # Get actual satellite position. For altitude use the ellipsoid radius at the SSP.
+        actual_lon = float(self.nav_info['SSP_longitude'])
+        actual_lat = float(self.nav_info['SSP_latitude'])
+        re = get_earth_radius(lon=actual_lon, lat=actual_lat,
+                              a=float(self.proj_info['earth_equatorial_radius'] * 1000),
+                              b=float(self.proj_info['earth_polar_radius'] * 1000))
+        actual_alt = float(self.nav_info['distance_earth_center_to_satellite']) * 1000 - re
+
         # Update metadata
-        new_info = dict(units=info['units'],
-                        standard_name=info['standard_name'],
-                        wavelength=info['wavelength'],
-                        resolution='resolution',
-                        id=key,
-                        name=key.name,
-                        scheduled_time=self.scheduled_time,
-                        platform_name=self.platform_name,
-                        sensor=self.sensor,
-                        satellite_longitude=float(
-                            self.nav_info['SSP_longitude']),
-                        satellite_latitude=float(
-                            self.nav_info['SSP_latitude']),
-                        satellite_altitude=float(self.nav_info['distance_earth_center_to_satellite'] -
-                                                 self.proj_info['earth_equatorial_radius']) * 1000)
+        new_info = dict(
+            units=info['units'],
+            standard_name=info['standard_name'],
+            wavelength=info['wavelength'],
+            resolution='resolution',
+            id=key,
+            name=key.name,
+            scheduled_time=self.scheduled_time,
+            platform_name=self.platform_name,
+            sensor=self.sensor,
+            satellite_longitude=float(self.nav_info['SSP_longitude']),
+            satellite_latitude=float(self.nav_info['SSP_latitude']),
+            satellite_altitude=float(self.nav_info['distance_earth_center_to_satellite'] -
+                                     self.proj_info['earth_equatorial_radius']) * 1000,
+            orbital_parameters={
+                'projection_longitude': float(self.proj_info['sub_lon']),
+                'projection_latitude': 0.,
+                'projection_altitude': float(self.proj_info['distance_from_earth_center'] -
+                                             self.proj_info['earth_equatorial_radius']) * 1000,
+                'satellite_actual_longitude': actual_lon,
+                'satellite_actual_latitude': actual_lat,
+                'satellite_actual_altitude': actual_alt,
+                'nadir_longitude': float(self.nav_info['nadir_longitude']),
+                'nadir_latitude': float(self.nav_info['nadir_latitude'])}
+        )
         res = xr.DataArray(res, attrs=new_info, dims=['y', 'x'])
 
         # Mask space pixels
-        res = self._mask_space(res)
+        if self.mask_space:
+            res = self._mask_space(res)
 
         return res
 
@@ -468,8 +566,19 @@ class AHIHSDFileHandler(BaseFileHandler):
     def convert_to_radiance(self, data):
         """Calibrate to radiance."""
 
-        gain = self._header["block5"]["gain_count2rad_conversion"][0]
-        offset = self._header["block5"]["offset_count2rad_conversion"][0]
+        bnum = self._header["block5"]['band_number'][0]
+        # Check calibration mode and select corresponding coefficients
+        if self.calib_mode == "UPDATE" and bnum < 7:
+            gain = self._header['calibration']["cali_gain_count2rad_conversion"][0]
+            offset = self._header['calibration']["cali_offset_count2rad_conversion"][0]
+            if gain == 0 and offset == 0:
+                logger.info(
+                    "No valid updated coefficients, fall back to default values.")
+                gain = self._header["block5"]["gain_count2rad_conversion"][0]
+                offset = self._header["block5"]["offset_count2rad_conversion"][0]
+        else:
+            gain = self._header["block5"]["gain_count2rad_conversion"][0]
+            offset = self._header["block5"]["offset_count2rad_conversion"][0]
 
         return (data * gain + offset).clip(0)
 
