@@ -1,25 +1,20 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-
-# Copyright (c) 2010-2017
-
-# Author(s):
-
-#   Martin Raspaud <martin.raspaud@smhi.se>
-
-# This program is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
-
+# Copyright (c) 2010-2017 Satpy developers
+#
+# This file is part of satpy.
+#
+# satpy is free software: you can redistribute it and/or modify it under the
+# terms of the GNU General Public License as published by the Free Software
+# Foundation, either version 3 of the License, or (at your option) any later
+# version.
+#
+# satpy is distributed in the hope that it will be useful, but WITHOUT ANY
+# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
+# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License along with
+# satpy.  If not, see <http://www.gnu.org/licenses/>.
 """HRIT format reader for JMA data
 ************************************
 
@@ -292,6 +287,10 @@ class HRITJMAFileHandler(HRITFileHandler):
         res.attrs['satellite_longitude'] = float(self.mda['projection_parameters']['SSP_longitude'])
         res.attrs['satellite_latitude'] = 0.
         res.attrs['satellite_altitude'] = float(self.mda['projection_parameters']['h'])
+        res.attrs['orbital_parameters'] = {
+            'projection_longitude': float(self.mda['projection_parameters']['SSP_longitude']),
+            'projection_latitude': 0.,
+            'projection_altitude': float(self.mda['projection_parameters']['h'])}
 
         return res
 
