@@ -1,27 +1,20 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# Copyright (c) 2011, 2012, 2013, 2014, 2015.
-
-# Author(s):
-
+# Copyright (c) 2011-2019 Satpy developers
 #
-#   David Hoese <david.hoese@ssec.wisc.edu>
-#
-
 # This file is part of satpy.
-
+#
 # satpy is free software: you can redistribute it and/or modify it under the
 # terms of the GNU General Public License as published by the Free Software
 # Foundation, either version 3 of the License, or (at your option) any later
 # version.
-
+#
 # satpy is distributed in the hope that it will be useful, but WITHOUT ANY
 # WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 # A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
-
+#
 # You should have received a copy of the GNU General Public License along with
 # satpy.  If not, see <http://www.gnu.org/licenses/>.
-
 """Interface to VIIRS L1B format
 
 """
@@ -36,6 +29,7 @@ LOG = logging.getLogger(__name__)
 class VIIRSL1BFileHandler(NetCDF4FileHandler):
     """VIIRS L1B File Reader
     """
+
     def _parse_datetime(self, datestr):
         return datetime.strptime(datestr, "%Y-%m-%dT%H:%M:%S.000Z")
 
@@ -59,13 +53,14 @@ class VIIRSL1BFileHandler(NetCDF4FileHandler):
             res = self.get('/attr/platform',
                            self.filename_info['platform_shortname'])
         except KeyError:
-            res = 'Suomi-NPP'
+            res = 'Unknown'
 
         return {
-            'Suomi-NPP': 'NPP',
-            'JPSS-1': 'J01',
-            'NP': 'NPP',
-            'J1': 'J01',
+            'JPSS-1': 'NOAA-20',
+            'NP': 'Suomi-NPP',
+            'J1': 'NOAA-20',
+            'J2': 'NOAA-21',
+            'JPSS-2': 'NOAA-21',
         }.get(res, res)
 
     @property
