@@ -1,24 +1,19 @@
 #!/usr/bin/python
-# Copyright (c) 2018.
+# Copyright (c) 2018 Satpy developers
 #
-
-# Author(s):
-#   Panu Lahtinen <panu.lahtinen@fmi.fi>
-
-# This program is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# This file is part of satpy.
 #
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-# General Public License for more details.
+# satpy is free software: you can redistribute it and/or modify it under the
+# terms of the GNU General Public License as published by the Free Software
+# Foundation, either version 3 of the License, or (at your option) any later
+# version.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program. If not, see <http://www.gnu.org/licenses/>.
+# satpy is distributed in the hope that it will be useful, but WITHOUT ANY
+# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
+# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #
-
+# You should have received a copy of the GNU General Public License along with
+# satpy.  If not, see <http://www.gnu.org/licenses/>.
 """Unittests for generic image reader.
 """
 
@@ -124,6 +119,7 @@ class TestGenericImage(unittest.TestCase):
         self.assertEqual(scn.attrs['sensor'], set(['images']))
         self.assertEqual(scn.attrs['start_time'], None)
         self.assertEqual(scn.attrs['end_time'], None)
+        self.assertNotIn('area', scn['image'].attrs)
 
         fname = os.path.join(self.base_dir, '20180101_0000_test_la.png')
         scn = Scene(reader='generic_image', filenames=[fname])
@@ -133,6 +129,7 @@ class TestGenericImage(unittest.TestCase):
         self.assertEqual(scn.attrs['sensor'], set(['images']))
         self.assertEqual(scn.attrs['start_time'], self.date)
         self.assertEqual(scn.attrs['end_time'], self.date)
+        self.assertNotIn('area', scn['image'].attrs)
         self.assertEqual(np.sum(np.isnan(data)), 100)
 
     def test_geotiff_scene(self):
