@@ -1,24 +1,20 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
+# Copyright (c) 2017 Satpy developers
 #
-# Copyright (c) 2017 David Hoese
+# This file is part of satpy.
 #
-# Author(s):
+# satpy is free software: you can redistribute it and/or modify it under the
+# terms of the GNU General Public License as published by the Free Software
+# Foundation, either version 3 of the License, or (at your option) any later
+# version.
 #
-#   David Hoese <david.hoese@ssec.wisc.edu>
+# satpy is distributed in the hope that it will be useful, but WITHOUT ANY
+# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
+# A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 #
-# This program is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# You should have received a copy of the GNU General Public License along with
+# satpy.  If not, see <http://www.gnu.org/licenses/>.
 """Tests for the geotiff writer.
 """
 import sys
@@ -97,6 +93,19 @@ class TestGeoTIFFWriter(unittest.TestCase):
             if hasattr(target, 'close'):
                 target.close()
 
+    def test_colormap_write(self):
+        """Test writing an image with a colormap."""
+        from satpy.writers.geotiff import GeoTIFFWriter
+        from trollimage.xrimage import XRImage
+        from trollimage.colormap import spectral
+        datasets = self._get_test_datasets()
+        w = GeoTIFFWriter(base_dir=self.base_dir)
+        # we'd have to customize enhancements to test this through
+        # save_datasets. We'll use `save_image` as a workaround.
+        img = XRImage(datasets[0])
+        img.palettize(spectral)
+        w.save_image(img, keep_palette=True)
+
     def test_float_write(self):
         """Test that geotiffs can be written as floats.
 
@@ -121,6 +130,18 @@ class TestGeoTIFFWriter(unittest.TestCase):
             w.save_datasets(datasets, compute=False)
             self.assertEqual(save_method.call_args[1]['fill_value'], 128)
 
+    def test_tags(self):
+        """Test tags being added."""
+        from satpy.writers.geotiff import GeoTIFFWriter
+        datasets = self._get_test_datasets()
+        w = GeoTIFFWriter(tags={'test1': 1}, base_dir=self.base_dir)
+        w.info['fill_value'] = 128
+        with mock.patch('satpy.writers.XRImage.save') as save_method:
+            save_method.return_value = None
+            w.save_datasets(datasets, tags={'test2': 2}, compute=False)
+            called_tags = save_method.call_args[1]['tags']
+            self.assertDictEqual(called_tags, {'test1': 1, 'test2': 2})
+
 
 def suite():
     """The test suite for this writer's tests."""
