@@ -1,23 +1,18 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# Copyright (c) 2015-2017.
-
-# Author(s):
-
-#   David Hoese <david.hoese@ssec.wisc.edu>
-#   Martin Raspaud <martin.raspaud@smhi.se>
-
+# Copyright (c) 2015-2019 Satpy developers
+#
 # This file is part of satpy.
-
+#
 # satpy is free software: you can redistribute it and/or modify it under the
 # terms of the GNU General Public License as published by the Free Software
 # Foundation, either version 3 of the License, or (at your option) any later
 # version.
-
+#
 # satpy is distributed in the hope that it will be useful, but WITHOUT ANY
 # WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 # A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
-
+#
 # You should have received a copy of the GNU General Public License along with
 # satpy.  If not, see <http://www.gnu.org/licenses/>.
 """Shared objects of the various writer classes.
@@ -27,13 +22,18 @@ For now, this includes enhancement configuration utilities.
 
 import logging
 import os
-
-import numpy as np
-import yaml
-import dask.array as da
-import xarray as xr
 import warnings
 
+import dask.array as da
+import numpy as np
+import xarray as xr
+import yaml
+
+try:
+    from yaml import UnsafeLoader
+except ImportError:
+    from yaml import Loader as UnsafeLoader
+
 from satpy.config import (config_search_paths, glob_config,
                           get_environ_config_dir, recursive_dict_update)
 from satpy import CHUNK_SIZE
@@ -47,14 +47,14 @@ from trollimage.xrimage import XRImage
 LOG = logging.getLogger(__name__)
 
 
-def read_writer_config(config_files, loader=yaml.Loader):
+def read_writer_config(config_files, loader=UnsafeLoader):
     """Read the writer `config_files` and return the info extracted."""
 
     conf = {}
     LOG.debug('Reading %s', str(config_files))
     for config_file in config_files:
         with open(config_file) as fd:
-            conf.update(yaml.load(fd.read(), loader))
+            conf.update(yaml.load(fd.read(), Loader=loader))
 
     try:
         writer_info = conf['writer']
@@ -179,8 +179,9 @@ def _determine_mode(dataset):
 
 
 def add_overlay(orig, area, coast_dir, color=(0, 0, 0), width=0.5, resolution=None,
-                level_coast=1, level_borders=1, fill_value=None):
-    """Add coastline and political borders to image.
+                level_coast=1, level_borders=1, fill_value=None,
+                grid=None):
+    """Add coastline, political borders and grid(graticules) to image.
 
     Uses ``color`` for feature colors where ``color`` is a 3-element tuple
     of integers between 0 and 255 representing (R, G, B).
@@ -189,16 +190,38 @@ def add_overlay(orig, area, coast_dir, color=(0, 0, 0), width=0.5, resolution=No
 
         This function currently loses the data mask (alpha band).
 
-    ``resolution`` is chosen automatically if None (default), otherwise it should be one of:
+    ``resolution`` is chosen automatically if None (default),
+    otherwise it should be one of:
 
     +-----+-------------------------+---------+
     | 'f' | Full resolution         | 0.04 km |
+    +-----+-------------------------+---------+
     | 'h' | High resolution         | 0.2 km  |
+    +-----+-------------------------+---------+
     | 'i' | Intermediate resolution | 1.0 km  |
+    +-----+-------------------------+---------+
     | 'l' | Low resolution          | 5.0 km  |
+    +-----+-------------------------+---------+
     | 'c' | Crude resolution        | 25  km  |
     +-----+-------------------------+---------+
 
+    ``grid`` is a dictionary with key values as documented in detail in pycoast
+
+    eg. overlay={'grid': {'major_lonlat': (10, 10),
+                          'write_text': False,
+                          'outline': (224, 224, 224),
+                          'width': 0.5}}
+
+    Here major_lonlat is plotted every 10 deg for both longitude and latitude,
+    no labels for the grid lines are plotted, the color used for the grid lines
+    is light gray, and the width of the gratucules is 0.5 pixels.
+
+    For grid if aggdraw is used, font option is mandatory, if not
+    ``write_text`` is set to False::
+
+        font = aggdraw.Font('black', '/usr/share/fonts/truetype/msttcorefonts/Arial.ttf',
+                            opacity=127, size=16)
+
     """
 
     if area is None:
@@ -244,6 +267,12 @@ def add_overlay(orig, area, coast_dir, color=(0, 0, 0), width=0.5, resolution=No
                        resolution=resolution, width=width, level=level_coast)
     cw_.add_borders(img, area, outline=color,
                     resolution=resolution, width=width, level=level_borders)
+    # Only add grid if major_lonlat is given.
+    if grid and 'major_lonlat' in grid and grid['major_lonlat']:
+        major_lonlat = grid.pop('major_lonlat')
+        minor_lonlat = grid.pop('minor_lonlat', major_lonlat)
+
+        cw_.add_grid(img, area, major_lonlat, minor_lonlat, **grid)
 
     arr = da.from_array(np.array(img) / 255.0, chunks=CHUNK_SIZE)
 
@@ -435,6 +464,19 @@ def show(dataset, **kwargs):
 
 
 def to_image(dataset):
+    """convert ``dataset`` into a :class:`~trollimage.xrimage.XRImage` instance.
+
+    Convert the ``dataset`` into an instance of the
+    :class:`~trollimage.xrimage.XRImage` class.  This function makes no other
+    changes.  To get an enhanced image, possibly with overlays and decoration,
+    see :func:`~get_enhanced_image`.
+
+    Args:
+        dataset (xarray.DataArray): Data to be converted to an image.
+
+    Returns:
+        Instance of :class:`~trollimage.xrimage.XRImage`.
+    """
     dataset = dataset.squeeze()
     if dataset.ndim < 2:
         raise ValueError("Need at least a 2D array to make an image.")
@@ -611,8 +653,8 @@ class Writer(Plugin):
                                  save using this writer.
             compute (bool): If `True` (default), compute all of the saves to
                             disk. If `False` then the return value is either
-                            a `dask.delayed.Delayed` object or two lists to
-                            be passed to a `dask.array.store` call.
+                            a :doc:`dask:delayed` object or two lists to
+                            be passed to a :func:`dask.array.store` call.
                             See return values below for more details.
             **kwargs: Keyword arguments to pass to `save_dataset`. See that
                       documentation for more details.
@@ -620,11 +662,11 @@ class Writer(Plugin):
         Returns:
             Value returned depends on `compute` keyword argument. If
             `compute` is `True` the value is the result of a either a
-            `dask.array.store` operation or a `dask.delayed.Delayed` compute,
-            typically this is `None`. If `compute` is `False` then the
-            result is either a `dask.delayed.Delayed` object that can be
+            :func:`dask.array.store` operation or a :doc:`dask:delayed`
+            compute, typically this is `None`. If `compute` is `False` then
+            the result is either a :doc:`dask:delayed` object that can be
             computed with `delayed.compute()` or a two element tuple of
-            sources and targets to be passed to `dask.array.store`. If
+            sources and targets to be passed to :func:`dask.array.store`. If
             `targets` is provided then it is the caller's responsibility to
             close any objects that have a "close" method.
 
@@ -660,7 +702,7 @@ class Writer(Plugin):
                                        with this fill value if applicable to
                                        this writer.
             compute (bool): If `True` (default), compute and save the dataset.
-                            If `False` return either a `dask.delayed.Delayed`
+                            If `False` return either a :doc:`dask:delayed`
                             object or tuple of (source, target). See the
                             return values below for more information.
             **kwargs: Other keyword arguments for this particular writer.
@@ -668,13 +710,13 @@ class Writer(Plugin):
         Returns:
             Value returned depends on `compute`. If `compute` is `True` then
             the return value is the result of computing a
-            `dask.delayed.Delayed` object or running `dask.array.store`. If
-            `compute` is `False` then the returned value is either a
-            `dask.delayed.Delayed` object that can be computed using
+            :doc:`dask:delayed` object or running :func:`dask.array.store`.
+            If `compute` is `False` then the returned value is either a
+            :doc:`dask:delayed` object that can be computed using
             `delayed.compute()` or a tuple of (source, target) that should be
-            passed to `dask.array.store`. If target is provided the the caller
-            is responsible for calling `target.close()` if the target has
-            this method.
+            passed to :func:`dask.array.store`. If target is provided the the
+            caller is responsible for calling `target.close()` if the target
+            has this method.
 
         """
         raise NotImplementedError(
@@ -755,9 +797,9 @@ class ImageWriter(Writer):
                      overlay=None, decorate=None, compute=True, **kwargs):
         """Saves the ``dataset`` to a given ``filename``.
 
-        This method creates an enhanced image using `get_enhanced_image`. The
-        image is then passed to `save_image`. See both of these functions for
-        more details on the arguments passed to this method.
+        This method creates an enhanced image using :func:`get_enhanced_image`.
+        The image is then passed to :meth:`save_image`. See both of these
+        functions for more details on the arguments passed to this method.
 
         """
         img = get_enhanced_image(dataset.squeeze(), enhance=self.enhancer, overlay=overlay,
@@ -774,7 +816,7 @@ class ImageWriter(Writer):
                             patterns that will be filled in by dataset
                             attributes.
             compute (bool): If `True` (default), compute and save the dataset.
-                            If `False` return either a `dask.delayed.Delayed`
+                            If `False` return either a :doc:`dask:delayed`
                             object or tuple of (source, target). See the
                             return values below for more information.
             **kwargs: Other keyword arguments to pass to this writer.
@@ -782,13 +824,13 @@ class ImageWriter(Writer):
         Returns:
             Value returned depends on `compute`. If `compute` is `True` then
             the return value is the result of computing a
-            `dask.delayed.Delayed` object or running `dask.array.store`. If
-            `compute` is `False` then the returned value is either a
-            `dask.delayed.Delayed` object that can be computed using
+            :doc:`dask:delayed` object or running :func:`dask.array.store`.
+            If `compute` is `False` then the returned value is either a
+            :doc:`dask:delayed` object that can be computed using
             `delayed.compute()` or a tuple of (source, target) that should be
-            passed to `dask.array.store`. If target is provided the the caller
-            is responsible for calling `target.close()` if the target has
-            this method.
+            passed to :func:`dask.array.store`. If target is provided the the
+            caller is responsible for calling `target.close()` if the target
+            has this method.
 
         """
         raise NotImplementedError("Writer '%s' has not implemented image saving" % (self.name,))
@@ -882,7 +924,7 @@ class EnhancementDecisionTree(DecisionTree):
         for config_file in decision_dict:
             if os.path.isfile(config_file):
                 with open(config_file) as fd:
-                    enhancement_config = yaml.load(fd)
+                    enhancement_config = yaml.load(fd, Loader=UnsafeLoader)
                     if enhancement_config is None:
                         # empty file
                         continue
@@ -896,7 +938,7 @@ class EnhancementDecisionTree(DecisionTree):
                 conf = recursive_dict_update(conf, config_file)
             else:
                 LOG.debug("Loading enhancement config string")
-                d = yaml.load(config_file)
+                d = yaml.load(config_file, Loader=UnsafeLoader)
                 if not isinstance(d, dict):
                     raise ValueError(
                         "YAML file doesn't exist or string is not YAML dict: {}".format(config_file))
